@@ -8,14 +8,20 @@ try:
 except:
     from urllib import urlencode  # noqa
 
+import django
 from django import forms
 from django.db.models.fields import BLANK_CHOICE_DASH
-from django.forms.widgets import flatatt
+
+from django.forms.utils import flatatt
+
+from django.utils.datastructures import MultiValueDict
 from django.utils.encoding import force_text
 from django.utils.safestring import mark_safe
 from django.utils.six import string_types
 from django.utils.translation import ugettext as _
 
+from .compat import format_value
+
 
 class LinkWidget(forms.Widget):
     def __init__(self, attrs=None, choices=()):
@@ -33,7 +39,10 @@ class LinkWidget(forms.Widget):
             self.data = {}
         if value is None:
             value = ''
-        final_attrs = self.build_attrs(attrs)
+        if django.VERSION < (1, 11):
+            final_attrs = self.build_attrs(attrs)
+        else:
+            final_attrs = self.build_attrs(self.attrs, extra_attrs=attrs)
         output = ['<ul%s>' % flatatt(final_attrs)]
         options = self.render_options(choices, [value], name)
         if options:
@@ -68,9 +77,9 @@ class LinkWidget(forms.Widget):
         except AttributeError:
             url = urlencode(data)
         return self.option_string() % {
-             'attrs': selected and ' class="selected"' or '',
-             'query_string': url,
-             'label': force_text(option_label)
+            'attrs': selected and ' class="selected"' or '',
+            'query_string': url,
+            'label': force_text(option_label)
         }
 
     def option_string(self):
@@ -78,18 +87,21 @@ class LinkWidget(forms.Widget):
 
 
 class RangeWidget(forms.MultiWidget):
+    template_name = 'django_filters/widgets/multiwidget.html'
+
     def __init__(self, attrs=None):
         widgets = (forms.TextInput, forms.TextInput)
         super(RangeWidget, self).__init__(widgets, attrs)
 
+    def format_output(self, rendered_widgets):
+        # Method was removed in Django 1.11.
+        return '-'.join(rendered_widgets)
+
     def decompress(self, value):
         if value:
             return [value.start, value.stop]
         return [None, None]
 
-    def format_output(self, rendered_widgets):
-        return '-'.join(rendered_widgets)
-
 
 class LookupTypeWidget(forms.MultiWidget):
     def decompress(self, value):
@@ -136,20 +148,68 @@ class BooleanWidget(forms.Select):
         }.get(value, None)
 
 
-class CSVWidget(forms.TextInput):
+class BaseCSVWidget(forms.Widget):
     def _isiterable(self, value):
         return isinstance(value, Iterable) and not isinstance(value, string_types)
 
     def value_from_datadict(self, data, files, name):
-        value = super(CSVWidget, self).value_from_datadict(data, files, name)
+        value = super(BaseCSVWidget, self).value_from_datadict(data, files, name)
 
         if value is not None:
+            if value == '':  # empty value should parse as an empty list
+                return []
             return value.split(',')
         return None
 
     def render(self, name, value, attrs=None):
-        if self._isiterable(value):
-            value = [force_text(self._format_value(v)) for v in value]
-            value = ','.join(list(value))
+        if not self._isiterable(value):
+            value = [value]
 
-        return super(CSVWidget, self).render(name, value, attrs)
+        if len(value) <= 1:
+            # delegate to main widget (Select, etc...) if not multiple values
+            value = value[0] if value else ''
+            return super(BaseCSVWidget, self).render(name, value, attrs)
+
+        # if we have multiple values, we need to force render as a text input
+        # (otherwise, the additional values are lost)
+        surrogate = forms.TextInput()
+        value = [force_text(format_value(surrogate, v)) for v in value]
+        value = ','.join(list(value))
+
+        return surrogate.render(name, value, attrs)
+
+
+class CSVWidget(BaseCSVWidget, forms.TextInput):
+    pass
+
+
+class QueryArrayWidget(BaseCSVWidget, forms.TextInput):
+    """
+    Enables request query array notation that might be consumed by MultipleChoiceFilter
+
+    1. Values can be provided as csv string:  ?foo=bar,baz
+    2. Values can be provided as query array: ?foo[]=bar&foo[]=baz
+    3. Values can be provided as query array: ?foo=bar&foo=baz
+
+    Note: Duplicate and empty values are skipped from results
+    """
+
+    def value_from_datadict(self, data, files, name):
+        if not isinstance(data, MultiValueDict):
+            for key, value in data.items():
+                # treat value as csv string: ?foo=1,2
+                if isinstance(value, string_types):
+                    data[key] = [x.strip() for x in value.rstrip(',').split(',') if x]
+            data = MultiValueDict(data)
+
+        values_list = data.getlist(name, data.getlist('%s[]' % name)) or []
+
+        # apparently its an array, so no need to process it's values as csv
+        # ?foo=1&foo=2 -> data.getlist(foo) -> foo = [1, 2]
+        # ?foo[]=1&foo[]=2 -> data.getlist(foo[]) -> foo = [1, 2]
+        if len(values_list) > 0:
+            ret = [x for x in values_list if x]
+        else:
+            ret = []
+
+        return list(set(ret))
