@@ -6,6 +6,7 @@ import mock
 import unittest
 
 import django
+from django import forms
 from django.test import TestCase, override_settings
 from django.utils import six
 from django.utils.timezone import now
@@ -13,28 +14,27 @@ from django.utils import timezone
 
 from django_filters.filterset import FilterSet
 from django_filters.filters import AllValuesFilter
-from django_filters.filters import BaseInFilter
+from django_filters.filters import AllValuesMultipleFilter
 from django_filters.filters import CharFilter
 from django_filters.filters import ChoiceFilter
 from django_filters.filters import DateRangeFilter
 from django_filters.filters import DateFromToRangeFilter
 from django_filters.filters import DateTimeFromToRangeFilter
-# from django_filters.filters import DateTimeFilter
-from django_filters.filters import MethodFilter
+from django_filters.filters import DurationFilter
 from django_filters.filters import MultipleChoiceFilter
+from django_filters.filters import ModelChoiceFilter
+from django_filters.filters import TypedMultipleChoiceFilter
 from django_filters.filters import ModelMultipleChoiceFilter
 from django_filters.filters import NumberFilter
+from django_filters.filters import OrderingFilter
 from django_filters.filters import RangeFilter
 from django_filters.filters import TimeRangeFilter
-# from django_filters.widgets import LinkWidget
+from django_filters.exceptions import FieldLookupError
 
 from .models import User
 from .models import Comment
 from .models import Book
-# from .models import Restaurant
 from .models import Article
-# from .models import NetworkSetting
-# from .models import SubnetMaskField
 from .models import Company
 from .models import Location
 from .models import Account
@@ -43,6 +43,7 @@ from .models import Profile
 from .models import Node
 from .models import DirectedNode
 from .models import STATUS_CHOICES
+from .models import SpacewalkRecord
 
 
 class CharFilterTests(TestCase):
@@ -125,12 +126,20 @@ class BooleanFilterTests(TestCase):
 
 class ChoiceFilterTests(TestCase):
 
-    def test_filtering(self):
+    @classmethod
+    def setUpTestData(cls):
         User.objects.create(username='alex', status=1)
         User.objects.create(username='jacob', status=2)
         User.objects.create(username='aaron', status=2)
         User.objects.create(username='carl', status=0)
 
+        Article.objects.create(author_id=1, published=now())
+        Article.objects.create(author_id=2, published=now())
+        Article.objects.create(author_id=3, published=now())
+        Article.objects.create(author_id=4, published=now())
+        Article.objects.create(author_id=None, published=now())
+
+    def test_filtering(self):
         class F(FilterSet):
             class Meta:
                 model = User
@@ -150,20 +159,15 @@ class ChoiceFilterTests(TestCase):
         f = F({'status': '0'})
         self.assertQuerysetEqual(f.qs, ['carl'], lambda o: o.username, False)
 
-
     def test_filtering_on_explicitly_defined_field(self):
         """
         Test for #30.
 
         If you explicitly declare ChoiceFilter fields you **MUST** pass `choices`.
         """
-        User.objects.create(username='alex', status=1)
-        User.objects.create(username='jacob', status=2)
-        User.objects.create(username='aaron', status=2)
-        User.objects.create(username='carl', status=0)
-
         class F(FilterSet):
             status = ChoiceFilter(choices=STATUS_CHOICES)
+
             class Meta:
                 model = User
                 fields = ['status']
@@ -182,6 +186,37 @@ class ChoiceFilterTests(TestCase):
         f = F({'status': '0'})
         self.assertQuerysetEqual(f.qs, ['carl'], lambda o: o.username, False)
 
+    def test_filtering_on_empty_choice(self):
+        class F(FilterSet):
+            class Meta:
+                model = User
+                fields = ['status']
+
+        f = F({'status': ''})
+        self.assertQuerysetEqual(f.qs,
+                                 ['aaron', 'alex', 'jacob', 'carl'],
+                                 lambda o: o.username, False)
+
+    def test_filtering_on_null_choice(self):
+        choices = [(u.pk, str(u)) for u in User.objects.order_by('id')]
+
+        class F(FilterSet):
+            author = ChoiceFilter(
+                choices=choices,
+                null_value='null',
+                null_label='NULL',
+            )
+
+            class Meta:
+                model = Article
+                fields = ['author']
+
+        # sanity check to make sure the filter is setup correctly
+        f = F({'author': '1'})
+        self.assertQuerysetEqual(f.qs, ['alex'], lambda o: str(o.author), False)
+
+        f = F({'author': 'null'})
+        self.assertQuerysetEqual(f.qs, [None], lambda o: o.author, False)
 
 
 class MultipleChoiceFilterTests(TestCase):
@@ -218,6 +253,41 @@ class MultipleChoiceFilterTests(TestCase):
             f.qs, ['aaron', 'alex', 'carl', 'jacob'], lambda o: o.username)
 
 
+class TypedMultipleChoiceFilterTests(TestCase):
+
+    def test_filtering(self):
+        User.objects.create(username='alex', status=1)
+        User.objects.create(username='jacob', status=2)
+        User.objects.create(username='aaron', status=2)
+        User.objects.create(username='carl', status=0)
+
+
+        class F(FilterSet):
+            status = TypedMultipleChoiceFilter(choices=STATUS_CHOICES, coerce=lambda x: x[0:2])
+
+            class Meta:
+                model = User
+                fields = ['status']
+
+        qs = User.objects.all().order_by('username')
+        f = F(queryset=qs)
+        self.assertQuerysetEqual(
+            f.qs, ['aa', 'ja', 'al', 'ca'],
+            lambda o: o.username[0:2], False)
+
+        f = F({'status': ['0']}, queryset=qs)
+        self.assertQuerysetEqual(
+            f.qs, ['ca'], lambda o: o.username[0:2])
+
+        f = F({'status': ['0', '1']}, queryset=qs)
+        self.assertQuerysetEqual(
+            f.qs, ['al', 'ca'], lambda o: o.username[0:2])
+
+        f = F({'status': ['0', '1', '2']}, queryset=qs)
+        self.assertQuerysetEqual(
+            f.qs, ['aa', 'al', 'ca', 'ja'], lambda o: o.username[0:2])
+
+
 class DateFilterTests(TestCase):
 
     def test_filtering(self):
@@ -300,6 +370,101 @@ class DateTimeFilterTests(TestCase):
         self.assertQuerysetEqual(f.qs, [2], lambda o: o.pk)
 
 
+class DurationFilterTests(TestCase):
+    """Duration filter tests.
+
+    The preferred format for durations in Django is '%d %H:%M:%S.%f'.
+    See django.utils.dateparse.parse_duration
+
+    Django is not fully ISO 8601 compliant (yet): year, month, and
+    week designators are not supported, so a duration string
+    like "P3Y6M4DT12H30M5S" cannot be used.
+    See https://en.wikipedia.org/wiki/ISO_8601#Durations
+
+    """
+    def setUp(self):
+        self.r1 = SpacewalkRecord.objects.create(
+            astronaut="Anatoly Solovyev",
+            duration=datetime.timedelta(hours=82, minutes=22))
+        self.r2 = SpacewalkRecord.objects.create(
+            astronaut="Michael Lopez-Alegria",
+            duration=datetime.timedelta(hours=67, minutes=40))
+        self.r3 = SpacewalkRecord.objects.create(
+            astronaut="Jerry L. Ross",
+            duration=datetime.timedelta(hours=58, minutes=32))
+        self.r4 = SpacewalkRecord.objects.create(
+            astronaut="John M. Grunsfeld",
+            duration=datetime.timedelta(hours=58, minutes=30))
+        self.r5 = SpacewalkRecord.objects.create(
+            astronaut="Richard Mastracchio",
+            duration=datetime.timedelta(hours=53, minutes=4))
+
+    def test_filtering(self):
+
+        class F(FilterSet):
+            class Meta:
+                model = SpacewalkRecord
+                fields = ['duration']
+
+        qs = SpacewalkRecord.objects.all()
+
+        # Django style: 3 days, 10 hours, 22 minutes.
+        f = F({'duration': '3 10:22:00'}, queryset=qs)
+        self.assertQuerysetEqual(f.qs, [self.r1], lambda x: x)
+
+        # ISO 8601: 3 days, 10 hours, 22 minutes.
+        f = F({'duration': 'P3DT10H22M'}, queryset=qs)
+        self.assertQuerysetEqual(f.qs, [self.r1], lambda x: x)
+
+        # Django style: 82 hours, 22 minutes.
+        f = F({'duration': '82:22:00'}, queryset=qs)
+        self.assertQuerysetEqual(f.qs, [self.r1], lambda x: x)
+
+        # ISO 8601: 82 hours, 22 minutes.
+        f = F({'duration': 'PT82H22M'}, queryset=qs)
+        self.assertQuerysetEqual(f.qs, [self.r1], lambda x: x)
+
+    def test_filtering_with_single_lookup_expr_dictionary(self):
+
+        class F(FilterSet):
+            class Meta:
+                model = SpacewalkRecord
+                fields = {'duration': ['gt', 'gte', 'lt', 'lte']}
+
+        qs = SpacewalkRecord.objects.order_by('-duration')
+
+        f = F({'duration__gt': 'PT58H30M'}, queryset=qs)
+        self.assertQuerysetEqual(
+            f.qs, [self.r1, self.r2, self.r3], lambda x: x)
+
+        f = F({'duration__gte': 'PT58H30M'}, queryset=qs)
+        self.assertQuerysetEqual(
+            f.qs, [self.r1, self.r2, self.r3, self.r4], lambda x: x)
+
+        f = F({'duration__lt': 'PT58H30M'}, queryset=qs)
+        self.assertQuerysetEqual(
+            f.qs, [self.r5], lambda x: x)
+
+        f = F({'duration__lte': 'PT58H30M'}, queryset=qs)
+        self.assertQuerysetEqual(
+            f.qs, [self.r4, self.r5], lambda x: x)
+
+    def test_filtering_with_multiple_lookup_exprs(self):
+
+        class F(FilterSet):
+            min_duration = DurationFilter(name='duration', lookup_expr='gte')
+            max_duration = DurationFilter(name='duration', lookup_expr='lte')
+
+            class Meta:
+                model = SpacewalkRecord
+                fields = '__all__'
+
+        qs = SpacewalkRecord.objects.order_by('duration')
+
+        f = F({'min_duration': 'PT55H', 'max_duration': 'PT60H'}, queryset=qs)
+        self.assertQuerysetEqual(f.qs, [self.r4, self.r3], lambda x: x)
+
+
 class ModelChoiceFilterTests(TestCase):
 
     def test_filtering(self):
@@ -320,6 +485,34 @@ class ModelChoiceFilterTests(TestCase):
         f = F({'author': jacob.pk}, queryset=qs)
         self.assertQuerysetEqual(f.qs, [1, 3], lambda o: o.pk, False)
 
+    def test_callable_queryset(self):
+        # Sanity check for callable queryset arguments.
+        # Ensure that nothing is improperly cached
+        User.objects.create(username='alex')
+        jacob = User.objects.create(username='jacob')
+        aaron = User.objects.create(username='aaron')
+
+        def users(request):
+            return User.objects.filter(pk__lt=request.user.pk)
+
+        class F(FilterSet):
+            author = ModelChoiceFilter(name='author', queryset=users)
+
+            class Meta:
+                model = Comment
+                fields = ['author']
+
+        qs = Comment.objects.all()
+        request = mock.Mock()
+
+        request.user = jacob
+        f = F(queryset=qs, request=request).filters['author'].field
+        self.assertQuerysetEqual(f.queryset, [1], lambda o: o.pk, False)
+
+        request.user = aaron
+        f = F(queryset=qs, request=request).filters['author'].field
+        self.assertQuerysetEqual(f.queryset, [1, 2], lambda o: o.pk, False)
+
 
 class ModelMultipleChoiceFilterTests(TestCase):
 
@@ -335,8 +528,8 @@ class ModelMultipleChoiceFilterTests(TestCase):
                                  average_rating=3.0)
         Book.objects.create(title="Stranger in a Strage Land", price='1.00',
                             average_rating=3.0)
-        alex.favorite_books = [b1, b2]
-        aaron.favorite_books = [b1, b3]
+        alex.favorite_books.add(b1, b2)
+        aaron.favorite_books.add(b1, b3)
 
         self.alex = alex
 
@@ -557,8 +750,13 @@ class RangeFilterTests(TestCase):
                                  lambda o: o.title)
 
 
-
-@unittest.skip('date-range is funky')
+# TODO:
+# year & month filtering could be better. The problem is that the test dates
+# are relative to today, which is always changing. So, two_weeks_ago is not a
+# valid date for 'this month' during the first half of the month, but is during
+# the second half. Similary, five_days_ago is not during 'this year' when the
+# tests are ran on January 1. All we can test is what is absolutely never valid
+# eg, a date from two_years_ago is never a valid date for 'this year'.
 class DateRangeFilterTests(TestCase):
 
     def setUp(self):
@@ -567,6 +765,7 @@ class DateRangeFilterTests(TestCase):
         two_weeks_ago = today - datetime.timedelta(days=14)
         two_months_ago = today - datetime.timedelta(days=62)
         two_years_ago = today - datetime.timedelta(days=800)
+
         alex = User.objects.create(username='alex')
         time = now().time()
         Comment.objects.create(date=two_weeks_ago, author=alex, time=time)
@@ -584,7 +783,10 @@ class DateRangeFilterTests(TestCase):
                 fields = ['date']
 
         f = F({'date': '4'})  # this year
-        self.assertQuerysetEqual(f.qs, [1, 3, 4, 5], lambda o: o.pk, False)
+
+        # assert what is NOT valid for now.
+        # self.assertQuerysetEqual(f.qs, [1, 3, 4, 5], lambda o: o.pk, False)
+        self.assertNotIn(2, f.qs.values_list('pk', flat=True))
 
     def test_filtering_for_month(self):
         class F(FilterSet):
@@ -595,9 +797,12 @@ class DateRangeFilterTests(TestCase):
                 fields = ['date']
 
         f = F({'date': '3'})  # this month
-        self.assertQuerysetEqual(f.qs, [1, 3, 4], lambda o: o.pk, False)
 
-    @unittest.expectedFailure
+        # assert what is NOT valid for now.
+        # self.assertQuerysetEqual(f.qs, [1, 3, 4], lambda o: o.pk, False)
+        self.assertNotIn(2, f.qs.values_list('pk', flat=True))
+        self.assertNotIn(5, f.qs.values_list('pk', flat=True))
+
     def test_filtering_for_week(self):
         class F(FilterSet):
             date = DateRangeFilter()
@@ -669,6 +874,92 @@ class DateFromToRangeFilterTests(TestCase):
             'published_1': '2016-01-03'})
         self.assertEqual(len(results.qs), 3)
 
+    @unittest.skipIf(django.VERSION < (1, 9), 'version doesnt supports is_dst parameter for make_aware')
+    @override_settings(TIME_ZONE='America/Sao_Paulo')
+    def test_filtering_dst_start_midnight(self):
+        tz = timezone.get_default_timezone()
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 10, 14, 23, 59)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 10, 15, 0, 0)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 10, 15, 1, 0)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 10, 16, 0, 0)))
+
+        class F(FilterSet):
+            published = DateFromToRangeFilter()
+
+            class Meta:
+                model = Article
+                fields = ['published']
+
+        results = F(data={
+            'published_0': '2017-10-15',
+            'published_1': '2017-10-15'})
+        self.assertEqual(len(results.qs), 2)
+
+    @unittest.skipIf(django.VERSION < (1, 9), 'version doesnt supports is_dst parameter for make_aware')
+    @override_settings(TIME_ZONE='America/Sao_Paulo')
+    def test_filtering_dst_ends_midnight(self):
+        tz = timezone.get_default_timezone()
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 2, 19, 0, 0)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 2, 18, 23, 0)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 2, 18, 0, 0)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 2, 17, 15, 0)))
+
+        class F(FilterSet):
+            published = DateFromToRangeFilter()
+
+            class Meta:
+                model = Article
+                fields = ['published']
+
+        results = F(data={
+            'published_0': '2017-02-18',
+            'published_1': '2017-02-18'})
+        self.assertEqual(len(results.qs), 2)
+
+    @unittest.skipIf(django.VERSION < (1, 9), 'version doesnt supports is_dst parameter for make_aware')
+    @override_settings(TIME_ZONE='Europe/Paris')
+    def test_filtering_dst_start(self):
+        tz = timezone.get_default_timezone()
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 3, 25, 23, 59)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 3, 26, 0, 0)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 3, 26, 2, 0)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 3, 26, 3, 0)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 3, 27, 0, 0)))
+
+        class F(FilterSet):
+            published = DateFromToRangeFilter()
+
+            class Meta:
+                model = Article
+                fields = ['published']
+
+        results = F(data={
+            'published_0': '2017-3-26',
+            'published_1': '2017-3-26'})
+        self.assertEqual(len(results.qs), 3)
+
+    @unittest.skipIf(django.VERSION < (1, 9), 'version doesnt supports is_dst parameter for make_aware')
+    @override_settings(TIME_ZONE='Europe/Paris')
+    def test_filtering_dst_end(self):
+        tz = timezone.get_default_timezone()
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 10, 28, 23, 59)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 10, 29, 0, 0)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 10, 29, 2, 0)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 10, 29, 3, 0)))
+        Article.objects.create(published=tz.localize(datetime.datetime(2017, 10, 30, 0, 0)))
+
+        class F(FilterSet):
+            published = DateFromToRangeFilter()
+
+            class Meta:
+                model = Article
+                fields = ['published']
+
+        results = F(data={
+            'published_0': '2017-10-29',
+            'published_1': '2017-10-29'})
+        self.assertEqual(len(results.qs), 3)
+
 
 class DateTimeFromToRangeFilterTests(TestCase):
 
@@ -735,9 +1026,9 @@ class AllValuesFilterTests(TestCase):
                 fields = ['username']
 
         self.assertEqual(list(F().qs), list(User.objects.all()))
-        self.assertEqual(list(F({'username': 'alex'})),
+        self.assertEqual(list(F({'username': 'alex'}).qs),
                          [User.objects.get(username='alex')])
-        self.assertEqual(list(F({'username': 'jose'})),
+        self.assertEqual(list(F({'username': 'jose'}).qs),
                          list())
 
     def test_filtering_without_strict(self):
@@ -747,20 +1038,20 @@ class AllValuesFilterTests(TestCase):
 
         class F(FilterSet):
             username = AllValuesFilter()
-            strict = False
 
             class Meta:
                 model = User
                 fields = ['username']
+                strict = False
 
         self.assertEqual(list(F().qs), list(User.objects.all()))
-        self.assertEqual(list(F({'username': 'alex'})),
+        self.assertEqual(list(F({'username': 'alex'}).qs),
                          [User.objects.get(username='alex')])
-        self.assertEqual(list(F({'username': 'jose'})),
+        self.assertEqual(list(F({'username': 'jose'}).qs),
                          list(User.objects.all()))
 
 
-class MethodFilterTests(TestCase):
+class AllValuesMultipleFilterTests(TestCase):
 
     def test_filtering(self):
         User.objects.create(username='alex')
@@ -768,96 +1059,61 @@ class MethodFilterTests(TestCase):
         User.objects.create(username='aaron')
 
         class F(FilterSet):
-            username = MethodFilter(action='filter_username')
+            username = AllValuesMultipleFilter()
 
             class Meta:
                 model = User
                 fields = ['username']
 
-            def filter_username(self, queryset, value):
-                return queryset.filter(
-                    username=value
-                )
-
         self.assertEqual(list(F().qs), list(User.objects.all()))
-        self.assertEqual(list(F({'username': 'alex'})),
+        self.assertEqual(list(F({'username': ['alex']}).qs),
                          [User.objects.get(username='alex')])
-        self.assertEqual(list(F({'username': 'jose'})),
+        self.assertEqual(list(F({'username': ['alex', 'jacob']}).qs),
+                         list(User.objects.filter(username__in=['alex', 'jacob'])))
+        self.assertEqual(list(F({'username': ['jose']}).qs),
                          list())
 
-    def test_filtering_external(self):
+
+class FilterMethodTests(TestCase):
+
+    def setUp(self):
         User.objects.create(username='alex')
         User.objects.create(username='jacob')
         User.objects.create(username='aaron')
 
-        def filter_username(queryset, value):
-            return queryset.filter(
-                username=value
-            )
-
+    def test_filtering(self):
         class F(FilterSet):
-            username = MethodFilter(action=filter_username)
+            username = CharFilter(method='filter_username')
 
             class Meta:
                 model = User
                 fields = ['username']
 
+            def filter_username(self, queryset, name, value):
+                return queryset.filter(**{name: value})
+
         self.assertEqual(list(F().qs), list(User.objects.all()))
-        self.assertEqual(list(F({'username': 'alex'})),
+        self.assertEqual(list(F({'username': 'alex'}).qs),
                          [User.objects.get(username='alex')])
-        self.assertEqual(list(F({'username': 'jose'})),
+        self.assertEqual(list(F({'username': 'jose'}).qs),
                          list())
 
-
-    def test_filtering_default_attribute_action(self):
-        User.objects.create(username='mike')
-        User.objects.create(username='jake')
-        User.objects.create(username='aaron')
+    def test_filtering_callable(self):
+        def filter_username(queryset, name, value):
+            return queryset.filter(**{name: value})
 
         class F(FilterSet):
-            username = MethodFilter()
-
-            class Meta:
-                model = User
-                fields = ['username']
-
-            def filter_username(self, queryset, value):
-                return queryset.filter(
-                    username__contains='ke'
-                )
-
-        self.assertEqual(list(F().qs), list(User.objects.all()))
-        self.assertEqual(list(F({'username': 'mike'})),
-                         [User.objects.get(username='mike'),
-                          User.objects.get(username='jake')],)
-        self.assertEqual(list(F({'username': 'jake'})),
-                         [User.objects.get(username='mike'),
-                          User.objects.get(username='jake')])
-        self.assertEqual(list(F({'username': 'aaron'})),
-                         [User.objects.get(username='mike'),
-                          User.objects.get(username='jake')])
-
-
-    def test_filtering_default(self):
-        User.objects.create(username='mike')
-        User.objects.create(username='jake')
-        User.objects.create(username='aaron')
-
-        class F(FilterSet):
-            username = MethodFilter()
-            email = MethodFilter()
+            username = CharFilter(method=filter_username)
 
             class Meta:
                 model = User
                 fields = ['username']
 
         self.assertEqual(list(F().qs), list(User.objects.all()))
-        self.assertEqual(list(F({'username': 'mike'})),
-                         list(User.objects.all()))
-        self.assertEqual(list(F({'username': 'jake'})),
-                         list(User.objects.all()))
-        self.assertEqual(list(F({'username': 'aaron'})),
-                         list(User.objects.all()))
+        self.assertEqual(list(F({'username': 'alex'}).qs),
+                         [User.objects.get(username='alex')])
+        self.assertEqual(list(F({'username': 'jose'}).qs),
+                         list())
 
 
 class O2ORelationshipTests(TestCase):
@@ -1120,8 +1376,8 @@ class M2MRelationshipTests(TestCase):
                                  average_rating=4.0)
         Book.objects.create(title="Stranger in a Strage Land", price='2.00',
                             average_rating=3.0)
-        alex.favorite_books = [b1, b2]
-        aaron.favorite_books = [b1, b3]
+        alex.favorite_books.add(b1, b2)
+        aaron.favorite_books.add(b1, b3)
 
     def test_m2m_relation(self):
         class F(FilterSet):
@@ -1414,56 +1670,50 @@ class CSVFilterTests(TestCase):
 
         qs = User.objects.all()
         f = F(queryset=qs)
-        self.assertEqual(len(f.qs), 4)
-        self.assertEqual(f.count(), 4)
+        self.assertEqual(f.qs.count(), 4)
 
         f = F({'status__in': ''}, queryset=qs)
-        self.assertEqual(len(f.qs), 0)
-        self.assertEqual(f.count(), 0)
+        self.assertEqual(f.qs.count(), 4)
+
+        f = F({'status__in': ','}, queryset=qs)
+        self.assertEqual(f.qs.count(), 0)
 
         f = F({'status__in': '0'}, queryset=qs)
-        self.assertEqual(len(f.qs), 1)
-        self.assertEqual(f.count(), 1)
+        self.assertEqual(f.qs.count(), 1)
 
         f = F({'status__in': '0,2'}, queryset=qs)
-        self.assertEqual(len(f.qs), 3)
-        self.assertEqual(f.count(), 3)
+        self.assertEqual(f.qs.count(), 3)
 
         f = F({'status__in': '0,,1'}, queryset=qs)
-        self.assertEqual(len(f.qs), 2)
-        self.assertEqual(f.count(), 2)
+        self.assertEqual(f.qs.count(), 2)
 
         f = F({'status__in': '2'}, queryset=qs)
-        self.assertEqual(len(f.qs), 2)
-        self.assertEqual(f.count(), 2)
+        self.assertEqual(f.qs.count(), 2)
 
     def test_string_filtering(self):
         F = self.user_filter
 
         qs = User.objects.all()
         f = F(queryset=qs)
-        self.assertEqual(len(f.qs), 4)
-        self.assertEqual(f.count(), 4)
+        self.assertEqual(f.qs.count(), 4)
 
         f = F({'username__in': ''}, queryset=qs)
-        self.assertEqual(len(f.qs), 0)
-        self.assertEqual(f.count(), 0)
+        self.assertEqual(f.qs.count(), 4)
+
+        f = F({'username__in': ','}, queryset=qs)
+        self.assertEqual(f.qs.count(), 0)
 
         f = F({'username__in': 'alex'}, queryset=qs)
-        self.assertEqual(len(f.qs), 1)
-        self.assertEqual(f.count(), 1)
+        self.assertEqual(f.qs.count(), 1)
 
         f = F({'username__in': 'alex,aaron'}, queryset=qs)
-        self.assertEqual(len(f.qs), 2)
-        self.assertEqual(f.count(), 2)
+        self.assertEqual(f.qs.count(), 2)
 
         f = F({'username__in': 'alex,,aaron'}, queryset=qs)
-        self.assertEqual(len(f.qs), 2)
-        self.assertEqual(f.count(), 2)
+        self.assertEqual(f.qs.count(), 2)
 
         f = F({'username__in': 'alex,'}, queryset=qs)
-        self.assertEqual(len(f.qs), 1)
-        self.assertEqual(f.count(), 1)
+        self.assertEqual(f.qs.count(), 1)
 
     def test_datetime_filtering(self):
         F = self.article_filter
@@ -1473,55 +1723,90 @@ class CSVFilterTests(TestCase):
         qs = Article.objects.all()
         f = F(queryset=qs)
         self.assertEqual(len(f.qs), 4)
-        self.assertEqual(f.count(), 4)
+        self.assertEqual(f.qs.count(), 4)
 
         f = F({'published__in': ''}, queryset=qs)
-        self.assertEqual(len(f.qs), 0)
-        self.assertEqual(f.count(), 0)
+        self.assertEqual(f.qs.count(), 4)
+
+        f = F({'published__in': ','}, queryset=qs)
+        self.assertEqual(f.qs.count(), 0)
 
         f = F({'published__in': '%s' % (after, )}, queryset=qs)
-        self.assertEqual(len(f.qs), 2)
-        self.assertEqual(f.count(), 2)
+        self.assertEqual(f.qs.count(), 2)
 
         f = F({'published__in': '%s,%s' % (after, before, )}, queryset=qs)
-        self.assertEqual(len(f.qs), 4)
-        self.assertEqual(f.count(), 4)
+        self.assertEqual(f.qs.count(), 4)
 
         f = F({'published__in': '%s,,%s' % (after, before, )}, queryset=qs)
-        self.assertEqual(len(f.qs), 4)
-        self.assertEqual(f.count(), 4)
+        self.assertEqual(f.qs.count(), 4)
 
         f = F({'published__in': '%s,' % (after, )}, queryset=qs)
-        self.assertEqual(len(f.qs), 2)
-        self.assertEqual(f.count(), 2)
+        self.assertEqual(f.qs.count(), 2)
 
     def test_related_filtering(self):
         F = self.article_filter
 
         qs = Article.objects.all()
         f = F(queryset=qs)
-        self.assertEqual(len(f.qs), 4)
-        self.assertEqual(f.count(), 4)
+        self.assertEqual(f.qs.count(), 4)
 
         f = F({'author__in': ''}, queryset=qs)
-        self.assertEqual(len(f.qs), 0)
-        self.assertEqual(f.count(), 0)
+        self.assertEqual(f.qs.count(), 4)
+
+        f = F({'author__in': ','}, queryset=qs)
+        self.assertEqual(f.qs.count(), 0)
 
         f = F({'author__in': '1'}, queryset=qs)
-        self.assertEqual(len(f.qs), 2)
-        self.assertEqual(f.count(), 2)
+        self.assertEqual(f.qs.count(), 2)
 
         f = F({'author__in': '1,2'}, queryset=qs)
-        self.assertEqual(len(f.qs), 4)
-        self.assertEqual(f.count(), 4)
+        self.assertEqual(f.qs.count(), 4)
 
         f = F({'author__in': '1,,2'}, queryset=qs)
-        self.assertEqual(len(f.qs), 4)
-        self.assertEqual(f.count(), 4)
+        self.assertEqual(f.qs.count(), 4)
 
         f = F({'author__in': '1,'}, queryset=qs)
-        self.assertEqual(len(f.qs), 2)
-        self.assertEqual(f.count(), 2)
+        self.assertEqual(f.qs.count(), 2)
+
+
+class OrderingFilterTests(TestCase):
+
+    def setUp(self):
+        User.objects.create(username='alex', status=1)
+        User.objects.create(username='jacob', status=2)
+        User.objects.create(username='aaron', status=2)
+        User.objects.create(username='carl', status=0)
+
+    def test_ordering(self):
+        class F(FilterSet):
+            o = OrderingFilter(
+                fields=('username', )
+            )
+
+            class Meta:
+                model = User
+                fields = ['username']
+
+        qs = User.objects.all()
+        f = F({'o': 'username'}, queryset=qs)
+        names = f.qs.values_list('username', flat=True)
+        self.assertEqual(list(names), ['aaron', 'alex', 'carl', 'jacob'])
+
+    def test_ordering_with_select_widget(self):
+        class F(FilterSet):
+            o = OrderingFilter(
+                widget=forms.Select,
+                fields=('username', )
+            )
+
+            class Meta:
+                model = User
+                fields = ['username']
+
+        qs = User.objects.all()
+        f = F({'o': 'username'}, queryset=qs)
+        names = f.qs.values_list('username', flat=True)
+        self.assertEqual(list(names), ['aaron', 'alex', 'carl', 'jacob'])
 
 
 class MiscFilterSetTests(TestCase):
@@ -1540,7 +1825,7 @@ class MiscFilterSetTests(TestCase):
                 model = User
                 fields = ['account']
 
-        qs = mock.MagicMock()
+        qs = mock.NonCallableMagicMock()
         f = F({'account': 'jdoe'}, queryset=qs)
         result = f.qs
         self.assertNotEqual(qs, result)
@@ -1560,20 +1845,9 @@ class MiscFilterSetTests(TestCase):
         f = F({'username': 'alex', 'status': '2'}, queryset=qs)
         self.assertQuerysetEqual(f.qs, [], lambda o: o.pk)
 
-    def test_filter_with_action(self):
-        class F(FilterSet):
-            username = CharFilter(action=lambda qs, value: (
-                qs.filter(**{'username__startswith': value})))
-
-            class Meta:
-                model = User
-                fields = ['username']
-
-        f = F({'username': 'a'}, queryset=User.objects.all())
-        self.assertQuerysetEqual(
-            f.qs, ['alex', 'aaron'], lambda o: o.username, False)
-
     def test_filter_with_initial(self):
+        # Initial values are a form presentation option - the FilterSet should
+        # not use an initial value as a default value to filter by.
         class F(FilterSet):
             status = ChoiceFilter(choices=STATUS_CHOICES, initial=1)
 
@@ -1582,8 +1856,10 @@ class MiscFilterSetTests(TestCase):
                 fields = ['status']
 
         qs = User.objects.all()
+        users = ['alex', 'jacob', 'aaron', 'carl']
+
         f = F(queryset=qs)
-        self.assertQuerysetEqual(f.qs, ['alex'], lambda o: o.username)
+        self.assertQuerysetEqual(f.qs.order_by('pk'), users, lambda o: o.username)
 
         f = F({'status': 0}, queryset=qs)
         self.assertQuerysetEqual(f.qs, ['carl'], lambda o: o.username)
@@ -1597,16 +1873,29 @@ class MiscFilterSetTests(TestCase):
         qs = User.objects.all()
         f = F(queryset=qs)
         self.assertEqual(len(f.qs), 4)
-        self.assertEqual(f.count(), 4)
+        self.assertEqual(f.qs.count(), 4)
 
         f = F({'status': '0'}, queryset=qs)
         self.assertEqual(len(f.qs), 1)
-        self.assertEqual(f.count(), 1)
+        self.assertEqual(f.qs.count(), 1)
 
         f = F({'status': '1'}, queryset=qs)
         self.assertEqual(len(f.qs), 1)
-        self.assertEqual(f.count(), 1)
+        self.assertEqual(f.qs.count(), 1)
 
         f = F({'status': '2'}, queryset=qs)
         self.assertEqual(len(f.qs), 2)
-        self.assertEqual(f.count(), 2)
+        self.assertEqual(f.qs.count(), 2)
+
+    def test_invalid_field_lookup(self):
+        # We want to ensure that non existent lookups (or just simple misspellings)
+        # throw a useful exception containg the field and lookup expr.
+        with self.assertRaises(FieldLookupError) as context:
+            class F(FilterSet):
+                class Meta:
+                    model = User
+                    fields = {'username': ['flub']}
+
+        exc = str(context.exception)
+        self.assertIn('tests.User.username', exc)
+        self.assertIn('flub', exc)
