@@ -1,15 +1,18 @@
+# -*- coding: utf-8 -*-
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
+from collections import OrderedDict
 from datetime import date, time, timedelta, datetime
+import inspect
 import mock
-import warnings
-import unittest
 
 from django import forms
 from django.test import TestCase, override_settings
+from django.utils import translation
+from django.utils.translation import ugettext as _
 
-from django_filters import filters
+from django_filters import filters, widgets
 from django_filters.fields import (
     Lookup,
     RangeField,
@@ -24,9 +27,11 @@ from django_filters.filters import (
     BooleanFilter,
     ChoiceFilter,
     MultipleChoiceFilter,
+    TypedMultipleChoiceFilter,
     DateFilter,
     DateTimeFilter,
     TimeFilter,
+    DurationFilter,
     ModelChoiceFilter,
     ModelMultipleChoiceFilter,
     NumberFilter,
@@ -41,11 +46,34 @@ from django_filters.filters import (
     BaseInFilter,
     BaseRangeFilter,
     UUIDFilter,
+    OrderingFilter,
     LOOKUP_TYPES)
 
 from tests.models import Book, User
 
 
+class ModuleImportTests(TestCase):
+    def is_filter(self, name, value):
+        return (
+            isinstance(value, type) and issubclass(value, Filter)
+        )
+
+    def test_imports(self):
+        # msg = "Expected `filters.%s` to be imported in `filters.__all__`"
+        filter_classes = [
+            key for key, value
+            in inspect.getmembers(filters)
+            if isinstance(value, type) and issubclass(value, Filter)
+        ]
+
+        # sanity check
+        self.assertIn('Filter', filter_classes)
+        self.assertIn('BooleanFilter', filter_classes)
+
+        for f in filter_classes:
+            self.assertIn(f, filters.__all__)
+
+
 class FilterTests(TestCase):
 
     def test_creation(self):
@@ -62,27 +90,6 @@ class FilterTests(TestCase):
         f = Filter()
         field = f.field
         self.assertIsInstance(field, forms.Field)
-        self.assertEqual(field.help_text, 'Filter')
-
-    def test_field_with_exclusion(self):
-        f = Filter(exclude=True)
-        field = f.field
-        self.assertIsInstance(field, forms.Field)
-        self.assertEqual(field.help_text, 'This is an exclusion filter')
-
-    @override_settings(FILTERS_HELP_TEXT_FILTER=False)
-    def test_default_field_settings(self):
-        f = Filter()
-        field = f.field
-        self.assertIsInstance(field, forms.Field)
-        self.assertEqual(field.help_text, '')
-
-    @override_settings(FILTERS_HELP_TEXT_EXCLUDE=False)
-    def test_field_with_exclusion_settings(self):
-        f = Filter(exclude=True)
-        field = f.field
-        self.assertIsInstance(field, forms.Field)
-        self.assertEqual(field.help_text, '')
 
     def test_field_with_single_lookup_expr(self):
         f = Filter(lookup_expr='iexact')
@@ -100,7 +107,6 @@ class FilterTests(TestCase):
         f = Filter(lookup_expr=None, exclude=True)
         field = f.field
         self.assertIsInstance(field, LookupTypeField)
-        self.assertEqual(field.help_text, 'This is an exclusion filter')
 
     def test_field_with_list_lookup_expr(self):
         f = Filter(lookup_expr=('istartswith', 'iendswith'))
@@ -111,29 +117,32 @@ class FilterTests(TestCase):
 
     def test_field_params(self):
         with mock.patch.object(Filter, 'field_class',
-                spec=['__call__']) as mocked:
+                               spec=['__call__']) as mocked:
             f = Filter(name='somefield', label='somelabel',
-                widget='somewidget')
+                       widget='somewidget')
             f.field
             mocked.assert_called_once_with(required=False,
-                label='somelabel', widget='somewidget', help_text=mock.ANY)
+                                           label='somelabel',
+                                           widget='somewidget')
 
     def test_field_extra_params(self):
         with mock.patch.object(Filter, 'field_class',
-                spec=['__call__']) as mocked:
+                               spec=['__call__']) as mocked:
             f = Filter(someattr='someattr')
             f.field
             mocked.assert_called_once_with(required=mock.ANY,
-                label=mock.ANY, widget=mock.ANY, help_text=mock.ANY,
-                someattr='someattr')
+                                           label=mock.ANY,
+                                           widget=mock.ANY,
+                                           someattr='someattr')
 
     def test_field_with_required_filter(self):
         with mock.patch.object(Filter, 'field_class',
-                spec=['__call__']) as mocked:
+                               spec=['__call__']) as mocked:
             f = Filter(required=True)
             f.field
             mocked.assert_called_once_with(required=True,
-                label=mock.ANY, widget=mock.ANY, help_text=mock.ANY)
+                                           label=mock.ANY,
+                                           widget=mock.ANY)
 
     def test_filtering(self):
         qs = mock.Mock(spec=['filter'])
@@ -192,12 +201,12 @@ class FilterTests(TestCase):
         qs.filter.assert_called_once_with(somefield__exact='value')
         self.assertNotEqual(qs, result)
 
-    def test_filter_using_action(self):
+    def test_filter_using_method(self):
         qs = mock.NonCallableMock(spec=[])
-        action = mock.Mock(spec=['filter'])
-        f = Filter(action=action)
+        method = mock.Mock()
+        f = Filter(method=method)
         result = f.filter(qs, 'value')
-        action.assert_called_once_with(qs, 'value')
+        method.assert_called_once_with(qs, None, 'value')
         self.assertNotEqual(qs, result)
 
     def test_filtering_uses_distinct(self):
@@ -207,16 +216,6 @@ class FilterTests(TestCase):
         result = qs.distinct.assert_called_once_with()
         self.assertNotEqual(qs, result)
 
-    def test_lookup_type_deprecation(self):
-        """
-        Make sure user is alerted when using deprecated ``lookup_type``.
-        """
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-            Filter(lookup_type='exact')
-            self.assertEqual(len(w), 1)
-            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
-
 
 class CustomFilterWithBooleanCheckTests(TestCase):
 
@@ -282,7 +281,6 @@ class BooleanFilterTests(TestCase):
         qs.exclude.assert_called_once_with(somefield__exact=True)
         self.assertNotEqual(qs, result)
 
-    @unittest.expectedFailure
     def test_filtering_skipped_with_blank_value(self):
         qs = mock.Mock()
         f = BooleanFilter(name='somefield')
@@ -312,6 +310,82 @@ class ChoiceFilterTests(TestCase):
         field = f.field
         self.assertIsInstance(field, forms.ChoiceField)
 
+    def test_empty_choice(self):
+        # default value
+        f = ChoiceFilter(choices=[('a', 'a')])
+        self.assertEqual(f.field.choices, [
+            ('', '---------'),
+            ('a', 'a'),
+        ])
+
+        # set value, allow blank label
+        f = ChoiceFilter(choices=[('a', 'a')], empty_label='')
+        self.assertEqual(f.field.choices, [
+            ('', ''),
+            ('a', 'a'),
+        ])
+
+        # disable empty choice w/ None
+        f = ChoiceFilter(choices=[('a', 'a')], empty_label=None)
+        self.assertEqual(f.field.choices, [
+            ('a', 'a'),
+        ])
+
+    def test_null_choice(self):
+        # default is to be disabled
+        f = ChoiceFilter(choices=[('a', 'a')], )
+        self.assertEqual(f.field.choices, [
+            ('', '---------'),
+            ('a', 'a'),
+        ])
+
+        # set label, allow blank label
+        f = ChoiceFilter(choices=[('a', 'a')], null_label='')
+        self.assertEqual(f.field.choices, [
+            ('', '---------'),
+            ('null', ''),
+            ('a', 'a'),
+        ])
+
+        # set null value
+        f = ChoiceFilter(choices=[('a', 'a')], null_value='NULL', null_label='')
+        self.assertEqual(f.field.choices, [
+            ('', '---------'),
+            ('NULL', ''),
+            ('a', 'a'),
+        ])
+
+        # explicitly disable
+        f = ChoiceFilter(choices=[('a', 'a')], null_label=None)
+        self.assertEqual(f.field.choices, [
+            ('', '---------'),
+            ('a', 'a'),
+        ])
+
+    @override_settings(
+        FILTERS_EMPTY_CHOICE_LABEL='EMPTY LABEL',
+        FILTERS_NULL_CHOICE_LABEL='NULL LABEL',
+        FILTERS_NULL_CHOICE_VALUE='NULL VALUE', )
+    def test_settings_overrides(self):
+        f = ChoiceFilter(choices=[('a', 'a')], )
+        self.assertEqual(f.field.choices, [
+            ('', 'EMPTY LABEL'),
+            ('NULL VALUE', 'NULL LABEL'),
+            ('a', 'a'),
+        ])
+
+    def test_callable_choices(self):
+        def choices():
+            yield ('a', 'a')
+            yield ('b', 'b')
+
+        f = ChoiceFilter(choices=choices)
+        self.assertEqual(f.field.choices, [
+            ('', '---------'),
+            ('a', 'a'),
+            ('b', 'b'),
+        ])
+
 
 class MultipleChoiceFilterTests(TestCase):
 
@@ -379,7 +453,6 @@ class MultipleChoiceFilterTests(TestCase):
         result = f.filter(qs, ['other', 'values', 'there'])
         self.assertEqual(qs, result)
 
-    @unittest.expectedFailure
     def test_filtering_skipped_with_empty_list_value_and_some_choices(self):
         qs = mock.Mock(spec=[])
         f = MultipleChoiceFilter(name='somefield')
@@ -437,7 +510,7 @@ class MultipleChoiceFilterTests(TestCase):
              [5, ]),
             ((books[3].pk, books[4].pk),
              []),
-            )
+        )
         users = User.objects.all()
 
         for item in filter_list:
@@ -451,6 +524,143 @@ class MultipleChoiceFilterTests(TestCase):
                     expected_pks, item[1], item[0]))
 
 
+class TypedMultipleChoiceFilterTests(TestCase):
+
+    def test_default_field(self):
+        f = TypedMultipleChoiceFilter()
+        field = f.field
+        self.assertIsInstance(field, forms.TypedMultipleChoiceField)
+
+    def test_filtering_requires_name(self):
+        qs = mock.Mock(spec=['filter'])
+        f = TypedMultipleChoiceFilter()
+        with self.assertRaises(TypeError):
+            f.filter(qs, ['value'])
+
+    def test_conjoined_default_value(self):
+        f = TypedMultipleChoiceFilter()
+        self.assertFalse(f.conjoined)
+
+    def test_conjoined_true(self):
+        f = TypedMultipleChoiceFilter(conjoined=True)
+        self.assertTrue(f.conjoined)
+
+    def test_filtering(self):
+        qs = mock.Mock(spec=['filter'])
+        f = TypedMultipleChoiceFilter(name='somefield')
+        with mock.patch('django_filters.filters.Q') as mockQclass:
+            mockQ1, mockQ2 = mock.MagicMock(), mock.MagicMock()
+            mockQclass.side_effect = [mockQ1, mockQ2]
+
+            f.filter(qs, ['value'])
+
+            self.assertEqual(mockQclass.call_args_list,
+                             [mock.call(), mock.call(somefield='value')])
+            mockQ1.__ior__.assert_called_once_with(mockQ2)
+            qs.filter.assert_called_once_with(mockQ1.__ior__.return_value)
+            qs.filter.return_value.distinct.assert_called_once_with()
+
+    def test_filtering_exclude(self):
+        qs = mock.Mock(spec=['exclude'])
+        f = TypedMultipleChoiceFilter(name='somefield', exclude=True)
+        with mock.patch('django_filters.filters.Q') as mockQclass:
+            mockQ1, mockQ2 = mock.MagicMock(), mock.MagicMock()
+            mockQclass.side_effect = [mockQ1, mockQ2]
+
+            f.filter(qs, ['value'])
+
+            self.assertEqual(mockQclass.call_args_list,
+                             [mock.call(), mock.call(somefield='value')])
+            mockQ1.__ior__.assert_called_once_with(mockQ2)
+            qs.exclude.assert_called_once_with(mockQ1.__ior__.return_value)
+            qs.exclude.return_value.distinct.assert_called_once_with()
+
+    def test_filtering_on_required_skipped_when_len_of_value_is_len_of_field_choices(self):
+        qs = mock.Mock(spec=[])
+        f = TypedMultipleChoiceFilter(name='somefield', required=True)
+        f.always_filter = False
+        result = f.filter(qs, [])
+        self.assertEqual(len(f.field.choices), 0)
+        self.assertEqual(qs, result)
+
+        f.field.choices = ['some', 'values', 'here']
+        result = f.filter(qs, ['some', 'values', 'here'])
+        self.assertEqual(qs, result)
+
+        result = f.filter(qs, ['other', 'values', 'there'])
+        self.assertEqual(qs, result)
+
+    def test_filtering_skipped_with_empty_list_value_and_some_choices(self):
+        qs = mock.Mock(spec=[])
+        f = TypedMultipleChoiceFilter(name='somefield')
+        f.field.choices = ['some', 'values', 'here']
+        result = f.filter(qs, [])
+        self.assertEqual(qs, result)
+
+    def test_filter_conjoined_true(self):
+        """Tests that a filter with `conjoined=True` returns objects that
+        have all the values included in `value`. For example filter
+        users that have all of this books.
+
+        """
+        book_kwargs = {'price': 1, 'average_rating': 1}
+        books = []
+        books.append(Book.objects.create(**book_kwargs))
+        books.append(Book.objects.create(**book_kwargs))
+        books.append(Book.objects.create(**book_kwargs))
+        books.append(Book.objects.create(**book_kwargs))
+        books.append(Book.objects.create(**book_kwargs))
+        books.append(Book.objects.create(**book_kwargs))
+
+        user1 = User.objects.create()
+        user2 = User.objects.create()
+        user3 = User.objects.create()
+        user4 = User.objects.create()
+        user5 = User.objects.create()
+
+        user1.favorite_books.add(books[0], books[1])
+        user2.favorite_books.add(books[0], books[1], books[2])
+        user3.favorite_books.add(books[1], books[2])
+        user4.favorite_books.add(books[2], books[3])
+        user5.favorite_books.add(books[4], books[5])
+
+        filter_list = (
+            ((books[0].pk, books[0].pk),  # values
+             [1, 2]),  # list of user.pk that have `value` books
+            ((books[1].pk, books[1].pk),
+             [1, 2, 3]),
+            ((books[2].pk, books[2].pk),
+             [2, 3, 4]),
+            ((books[3].pk, books[3].pk),
+             [4, ]),
+            ((books[4].pk, books[4].pk),
+             [5, ]),
+            ((books[0].pk, books[1].pk),
+             [1, 2]),
+            ((books[0].pk, books[2].pk),
+             [2, ]),
+            ((books[1].pk, books[2].pk),
+             [2, 3]),
+            ((books[2].pk, books[3].pk),
+             [4, ]),
+            ((books[4].pk, books[5].pk),
+             [5, ]),
+            ((books[3].pk, books[4].pk),
+             []),
+        )
+        users = User.objects.all()
+
+        for item in filter_list:
+            f = TypedMultipleChoiceFilter(name='favorite_books__pk', conjoined=True)
+            queryset = f.filter(users, item[0])
+            expected_pks = [c[0] for c in queryset.values_list('pk')]
+            self.assertListEqual(
+                expected_pks,
+                item[1],
+                'Lists Differ: {0} != {1} for case {2}'.format(
+                    expected_pks, item[1], item[0]))
+
+
 class DateFilterTests(TestCase):
 
     def test_default_field(self):
@@ -475,6 +685,14 @@ class TimeFilterTests(TestCase):
         self.assertIsInstance(field, forms.TimeField)
 
 
+class DurationFilterTests(TestCase):
+
+    def test_default_field(self):
+        f = DurationFilter()
+        field = f.field
+        self.assertIsInstance(field, forms.DurationField)
+
+
 class ModelChoiceFilterTests(TestCase):
 
     def test_default_field_without_queryset(self):
@@ -489,6 +707,33 @@ class ModelChoiceFilterTests(TestCase):
         self.assertIsInstance(field, forms.ModelChoiceField)
         self.assertEqual(field.queryset, qs)
 
+    def test_callable_queryset(self):
+        request = mock.NonCallableMock(spec=[])
+        qs = mock.NonCallableMock(spec=[])
+
+        qs_callable = mock.Mock(return_value=qs)
+
+        f = ModelChoiceFilter(queryset=qs_callable)
+        f.parent = mock.Mock(request=request)
+        field = f.field
+
+        qs_callable.assert_called_with(request)
+        self.assertEqual(field.queryset, qs)
+
+    def test_get_queryset_override(self):
+        request = mock.NonCallableMock(spec=[])
+        qs = mock.NonCallableMock(spec=[])
+
+        class F(ModelChoiceFilter):
+            get_queryset = mock.create_autospec(ModelChoiceFilter.get_queryset, return_value=qs)
+
+        f = F()
+        f.parent = mock.Mock(request=request)
+        field = f.field
+
+        f.get_queryset.assert_called_with(f, request)
+        self.assertEqual(field.queryset, qs)
+
 
 class ModelMultipleChoiceFilterTests(TestCase):
 
@@ -504,6 +749,34 @@ class ModelMultipleChoiceFilterTests(TestCase):
         self.assertIsInstance(field, forms.ModelMultipleChoiceField)
         self.assertEqual(field.queryset, qs)
 
+    def test_filtering_to_field_name(self):
+        qs = User.objects.all()
+        f = ModelMultipleChoiceFilter(name='first_name',
+                                      to_field_name='first_name',
+                                      queryset=qs)
+        user = User.objects.create(first_name='Firstname')
+
+        self.assertEqual(f.get_filter_predicate(user),
+                         {'first_name': 'Firstname'})
+        self.assertEqual(f.get_filter_predicate('FilterValue'),
+                         {'first_name': 'FilterValue'})
+
+        self.assertEqual(list(f.filter(qs, ['Firstname'])), [user])
+        self.assertEqual(list(f.filter(qs, [user])), [user])
+
+    def test_callable_queryset(self):
+        request = mock.NonCallableMock(spec=[])
+        qs = mock.NonCallableMock(spec=[])
+
+        qs_callable = mock.Mock(return_value=qs)
+
+        f = ModelMultipleChoiceFilter(queryset=qs_callable)
+        f.parent = mock.Mock(request=request)
+        field = f.field
+
+        qs_callable.assert_called_with(request)
+        self.assertEqual(field.queryset, qs)
+
 
 class NumberFilterTests(TestCase):
 
@@ -567,14 +840,6 @@ class NumericRangeFilterTests(TestCase):
         f.filter(qs, value)
         qs.filter.assert_called_once_with(None__overlap=(20, 30))
 
-    @unittest.expectedFailure
-    def test_filtering_lower_field_higher_than_upper_field(self):
-        qs = mock.Mock(spec=['filter'])
-        value = mock.Mock(start=35, stop=30)
-        f = NumericRangeFilter()
-        result = f.filter(qs, value)
-        self.assertEqual(qs, result)
-
     def test_zero_to_zero(self):
         qs = mock.Mock(spec=['filter'])
         value = mock.Mock(start=0, stop=0)
@@ -582,6 +847,20 @@ class NumericRangeFilterTests(TestCase):
         f.filter(qs, value)
         qs.filter.assert_called_once_with(None__exact=(0, 0))
 
+    def test_filtering_startswith(self):
+        qs = mock.Mock(spec=['filter'])
+        value = mock.Mock(start=20, stop=None)
+        f = NumericRangeFilter()
+        f.filter(qs, value)
+        qs.filter.assert_called_once_with(None__startswith=20)
+
+    def test_filtering_endswith(self):
+        qs = mock.Mock(spec=['filter'])
+        value = mock.Mock(start=None, stop=30)
+        f = NumericRangeFilter()
+        f.filter(qs, value)
+        qs.filter.assert_called_once_with(None__endswith=30)
+
 
 class RangeFilterTests(TestCase):
 
@@ -645,25 +924,18 @@ class DateRangeFilterTests(TestCase):
         self.assertIsInstance(field, forms.ChoiceField)
 
     def test_filtering(self):
-        qs = mock.Mock(spec=['all'])
-        f = DateRangeFilter()
-        f.filter(qs, '')
-        qs.all.assert_called_once_with()
-
-    # the correct behavior fails right now
-    @unittest.expectedFailure
-    def test_filtering_skipped_with_blank_value(self):
+        # skip filtering, as it's an empty value
         qs = mock.Mock(spec=[])
         f = DateRangeFilter()
         result = f.filter(qs, '')
         self.assertEqual(qs, result)
 
-    @unittest.expectedFailure
     def test_filtering_skipped_with_out_of_range_value(self):
+        # Field validation should prevent this from occuring
         qs = mock.Mock(spec=[])
         f = DateRangeFilter()
-        result = f.filter(qs, 999)
-        self.assertEqual(qs, result)
+        with self.assertRaises(AssertionError):
+            f.filter(qs, 999)
 
     def test_filtering_for_this_year(self):
         qs = mock.Mock(spec=['filter'])
@@ -685,18 +957,18 @@ class DateRangeFilterTests(TestCase):
 
     def test_filtering_for_7_days(self):
         qs = mock.Mock(spec=['filter'])
-        with mock.patch('django_filters.filters.now'):
-            with mock.patch('django_filters.filters.timedelta') as mock_td:
-                with mock.patch(
-                        'django_filters.filters._truncate') as mock_truncate:
-                    mock_dt1, mock_dt2 = mock.MagicMock(), mock.MagicMock()
-                    mock_truncate.side_effect = [mock_dt1, mock_dt2]
-                    f = DateRangeFilter()
-                    f.filter(qs, '2')
-                    self.assertEqual(mock_td.call_args_list,
-                        [mock.call(days=7), mock.call(days=1)])
-                    qs.filter.assert_called_once_with(
-                        None__lt=mock_dt2, None__gte=mock_dt1)
+        with mock.patch('django_filters.filters.now'), \
+                mock.patch('django_filters.filters.timedelta') as mock_td, \
+                mock.patch('django_filters.filters._truncate') as mock_truncate:
+            mock_d1, mock_d2 = mock.MagicMock(), mock.MagicMock()
+            mock_truncate.side_effect = [mock_d1, mock_d2]
+            f = DateRangeFilter()
+            f.filter(qs, '2')
+            self.assertEqual(
+                mock_td.call_args_list,
+                [mock.call(days=7), mock.call(days=1)]
+            )
+            qs.filter.assert_called_once_with(None__lt=mock_d2, None__gte=mock_d1)
 
     def test_filtering_for_today(self):
         qs = mock.Mock(spec=['filter'])
@@ -802,11 +1074,11 @@ class DateTimeFromToRangeFilterTests(TestCase):
         result = f.filter(qs, None)
         self.assertEqual(qs, result)
 
-    def test_filtering_ignores_lookup_type(self):
+    def test_filtering_ignores_lookup_expr(self):
         qs = mock.Mock()
         value = mock.Mock(
             start=datetime(2015, 4, 7, 8, 30), stop=datetime(2015, 9, 6, 11, 45))
-        f = DateTimeFromToRangeFilter(lookup_type='gte')
+        f = DateTimeFromToRangeFilter(lookup_expr='gte')
         f.filter(qs, value)
         qs.filter.assert_called_once_with(
             None__range=(datetime(2015, 4, 7, 8, 30), datetime(2015, 9, 6, 11, 45)))
@@ -866,7 +1138,8 @@ class AllValuesFilterTests(TestCase):
     def test_default_field_with_assigning_model(self):
         mocked = mock.Mock()
         chained_call = '.'.join(['_default_manager', 'distinct.return_value',
-            'order_by.return_value', 'values_list.return_value'])
+                                 'order_by.return_value',
+                                 'values_list.return_value'])
         mocked.configure_mock(**{chained_call: iter([])})
         f = AllValuesFilter()
         f.model = mocked
@@ -984,3 +1257,157 @@ class BaseRangeFilterTests(TestCase):
         f = NumberInFilter()
         f.filter(qs, [1, 2])
         qs.filter.assert_called_once_with(None__range=[1, 2])
+
+
+class OrderingFilterTests(TestCase):
+    def test_default_field(self):
+        f = OrderingFilter()
+        field = f.field
+        self.assertIsInstance(field, forms.ChoiceField)
+
+    def test_filtering(self):
+        qs = mock.Mock(spec=['order_by'])
+        f = OrderingFilter()
+        f.filter(qs, ['a', 'b'])
+        qs.order_by.assert_called_once_with('a', 'b')
+
+    def test_filtering_descending(self):
+        qs = mock.Mock(spec=['order_by'])
+        f = OrderingFilter()
+        f.filter(qs, ['-a'])
+        qs.order_by.assert_called_once_with('-a')
+
+    def test_filtering_with_fields(self):
+        qs = mock.Mock(spec=['order_by'])
+        f = OrderingFilter(fields={'a': 'b'})
+        f.filter(qs, ['b', '-b'])
+        qs.order_by.assert_called_once_with('a', '-a')
+
+    def test_filtering_skipped_with_none_value(self):
+        qs = mock.Mock(spec=['order_by'])
+        f = OrderingFilter()
+        result = f.filter(qs, None)
+        self.assertEqual(qs, result)
+
+    def test_choices_unaltered(self):
+        # provided 'choices' should not be altered when 'fields' is present
+        f = OrderingFilter(
+            choices=(('a', 'A'), ('b', 'B')),
+            fields=(('a', 'c'), ('b', 'd')),
+        )
+
+        self.assertSequenceEqual(f.field.choices, (
+            ('', '---------'),
+            ('a', 'A'),
+            ('b', 'B'),
+        ))
+
+    def test_choices_from_fields(self):
+        f = OrderingFilter(
+            fields=(('a', 'c'), ('b', 'd')),
+        )
+
+        self.assertSequenceEqual(f.field.choices, (
+            ('', '---------'),
+            ('c', 'C'),
+            ('-c', 'C (descending)'),
+            ('d', 'D'),
+            ('-d', 'D (descending)'),
+        ))
+
+    def test_field_labels(self):
+        f = OrderingFilter(
+            fields=(('a', 'c'), ('b', 'd')),
+            field_labels={'a': 'foo'},
+        )
+
+        self.assertSequenceEqual(f.field.choices, (
+            ('', '---------'),
+            ('c', 'foo'),
+            ('-c', 'foo (descending)'),
+            ('d', 'D'),
+            ('-d', 'D (descending)'),
+        ))
+
+    def test_field_labels_descending(self):
+        f = OrderingFilter(
+            fields=['username'],
+            field_labels={
+                'username': 'BLABLA',
+                '-username': 'XYZXYZ',
+            }
+        )
+
+        self.assertEqual(f.field.choices, [
+            ('', '---------'),
+            ('username', 'BLABLA'),
+            ('-username', 'XYZXYZ'),
+        ])
+
+    def test_normalize_fields(self):
+        f = OrderingFilter.normalize_fields
+        O = OrderedDict
+
+        self.assertIn('a', f({'a': 'b'}))
+
+        self.assertEqual(
+            f(O([('a', 'b'), ('c', 'd')])),
+            O([('a', 'b'), ('c', 'd')])
+        )
+
+        self.assertEqual(
+            f([('a', 'b'), ('c', 'd')]),
+            O([('a', 'b'), ('c', 'd')])
+        )
+
+        self.assertEqual(
+            f(['a', 'b']),
+            O([('a', 'a'), ('b', 'b')])
+        )
+
+        with self.assertRaises(AssertionError) as ctx:
+            f(None)
+        self.assertEqual(str(ctx.exception), "'fields' must be an iterable (e.g., a list, tuple, or mapping).")
+
+        with self.assertRaises(AssertionError) as ctx:
+            f([('a', 'b', 'c')])
+        self.assertEqual(str(ctx.exception), "'fields' must contain strings or (field name, param name) pairs.")
+
+        with self.assertRaises(AssertionError) as ctx:
+            f([0, 1, 2])
+        self.assertEqual(str(ctx.exception), "'fields' must contain strings or (field name, param name) pairs.")
+
+    def test_widget(self):
+        f = OrderingFilter()
+        widget = f.field.widget
+
+        self.assertIsInstance(widget, widgets.BaseCSVWidget)
+        self.assertIsInstance(widget, forms.Select)
+
+    def test_translation_sanity(self):
+        with translation.override('pl'):
+            self.assertEqual(_('Username'), 'Nazwa użytkownika')
+            self.assertEqual(_('%s (descending)') % _('Username'), 'Nazwa użytkownika (malejąco)')
+
+    def test_translation_default_label(self):
+        with translation.override('pl'):
+            f = OrderingFilter(fields=['username'])
+
+            self.assertEqual(f.field.choices, [
+                ('', '---------'),
+                ('username', 'Nazwa użytkownika'),
+                ('-username', 'Nazwa użytkownika (malejąco)'),
+            ])
+
+    def test_translation_override_label(self):
+        with translation.override('pl'):
+            f = OrderingFilter(
+                fields=['username'],
+                field_labels={'username': 'BLABLA'},
+            )
+
+            self.assertEqual(f.field.choices, [
+                ('', '---------'),
+                ('username', 'BLABLA'),
+                ('-username', 'BLABLA (malejąco)'),
+            ])
