@@ -2,15 +2,15 @@ from __future__ import absolute_import
 from __future__ import unicode_literals
 
 from django import forms
-from django.test import TestCase
+from django.test import TestCase, override_settings
 
 from django_filters.filterset import FilterSet
 from django_filters.filters import CharFilter
 from django_filters.filters import ChoiceFilter
 
-from .models import User
+from .models import User, ManagerGroup
 from .models import Book
-from .models import STATUS_CHOICES
+from .models import STATUS_CHOICES, REGULAR, MANAGER
 
 
 class FilterSetFormTests(TestCase):
@@ -39,6 +39,7 @@ class FilterSetFormTests(TestCase):
         class F(FilterSet):
             class Meta:
                 model = Book
+                fields = '__all__'
                 form = MyForm
 
         f = F().form
@@ -65,8 +66,10 @@ class FilterSetFormTests(TestCase):
         f = F().form
         self.assertEqual(len(f.fields), 1)
         self.assertIn('status', f.fields)
-        self.assertEqual(sorted(f.fields['status'].choices),
-                         sorted(STATUS_CHOICES))
+        self.assertSequenceEqual(
+            f.fields['status'].choices,
+            (('', '---------'), ) + STATUS_CHOICES
+        )
 
     def test_form_fields_exclusion(self):
         class F(FilterSet):
@@ -77,12 +80,36 @@ class FilterSetFormTests(TestCase):
                 fields = ('title',)
 
         f = F().form
-        self.assertEqual(f.fields['title'].help_text, "This is an exclusion filter")
+        self.assertEqual(f.fields['title'].label, "Exclude title")
+
+    def test_complex_form_fields(self):
+        class F(FilterSet):
+            username = CharFilter(label='Filter for users with username')
+            exclude_username = CharFilter(name='username', lookup_expr='iexact', exclude=True)
+
+            class Meta:
+                model = User
+                fields = {
+                    'status': ['exact', 'lt', 'gt'],
+                    'favorite_books__title': ['iexact', 'in'],
+                    'manager_of__users__username': ['exact'],
+                }
+
+        fields = F().form.fields
+        self.assertEqual(fields['username'].label, 'Filter for users with username')
+        self.assertEqual(fields['exclude_username'].label, 'Exclude username')
+        self.assertEqual(fields['status'].label, 'Status')
+        self.assertEqual(fields['status__lt'].label, 'Status is less than')
+        self.assertEqual(fields['status__gt'].label, 'Status is greater than')
+        self.assertEqual(fields['favorite_books__title__iexact'].label, 'Favorite books title')
+        self.assertEqual(fields['favorite_books__title__in'].label, 'Favorite books title is in')
+        self.assertEqual(fields['manager_of__users__username'].label, 'Manager of users username')
 
     def test_form_fields_using_widget(self):
         class F(FilterSet):
             status = ChoiceFilter(widget=forms.RadioSelect,
-                                  choices=STATUS_CHOICES)
+                                  choices=STATUS_CHOICES,
+                                  empty_label=None)
 
             class Meta:
                 model = User
@@ -92,8 +119,10 @@ class FilterSetFormTests(TestCase):
         self.assertEqual(len(f.fields), 2)
         self.assertIn('status', f.fields)
         self.assertIn('username', f.fields)
-        self.assertEqual(sorted(f.fields['status'].choices),
-                         sorted(STATUS_CHOICES))
+        self.assertSequenceEqual(
+            f.fields['status'].choices,
+            STATUS_CHOICES
+        )
         self.assertIsInstance(f.fields['status'].widget, forms.RadioSelect)
 
     def test_form_field_with_custom_label(self):
@@ -117,8 +146,8 @@ class FilterSetFormTests(TestCase):
                 fields = ('book_title',)
 
         f = F().form
-        self.assertEqual(f.fields['book_title'].label, None)
-        self.assertEqual(f['book_title'].label, 'Book title')
+        self.assertEqual(f.fields['book_title'].label, "Title")
+        self.assertEqual(f['book_title'].label, "Title")
 
     def test_form_field_with_manual_name_and_label(self):
         class F(FilterSet):
@@ -163,112 +192,40 @@ class FilterSetFormTests(TestCase):
         self.assertTrue(f.is_bound)
         self.assertEqual(f.data, {'title': 'Some book'})
 
-    def test_ordering(self):
+    def test_limit_choices_to(self):
+        User.objects.create(username='inactive', is_active=False, status=REGULAR)
+        User.objects.create(username='active', is_active=True, status=REGULAR)
+        User.objects.create(username='manager', is_active=False, status=MANAGER)
+
         class F(FilterSet):
             class Meta:
-                model = User
-                fields = ['username', 'status']
-                order_by = ['status']
-
-        f = F().form
-        self.assertEqual(len(f.fields), 3)
-        self.assertIn('o', f.fields)
-        self.assertEqual(f.fields['o'].choices, [('status', 'Status')])
-
-    def test_ordering_uses_all_fields(self):
-        class F(FilterSet):
-            class Meta:
-                model = User
-                fields = ['username', 'status']
-                order_by = True
-
-        f = F().form
-        self.assertEqual(f.fields['o'].choices,
-            [('username', 'Username'), ('-username', 'Username (descending)'), ('status', 'Status'), ('-status', 'Status (descending)')])
-
-    def test_ordering_uses_filter_label(self):
-        class F(FilterSet):
-            username = CharFilter(label='Account')
-
-            class Meta:
-                model = User
-                fields = ['username', 'status']
-                order_by = True
-
-        f = F().form
-        self.assertEqual(f.fields['o'].choices,
-            [('username', 'Account'), ('-username', 'Account (descending)'), ('status', 'Status'), ('-status', 'Status (descending)')])
-
-    def test_ordering_uses_explicit_filter_name(self):
-        class F(FilterSet):
-            account = CharFilter(name='username')
-
-            class Meta:
-                model = User
-                fields = ['account', 'status']
-                order_by = True
-
-        f = F().form
-        self.assertEqual(f.fields['o'].choices,
-            [('account', 'Account'), ('-account', 'Account (descending)'), ('status', 'Status'), ('-status', 'Status (descending)')])
-
-    def test_ordering_with_overridden_field_name(self):
-        """
-        Set the `order_by_field` on the queryset and ensure that the
-        field name is respected.
-        """
-        class F(FilterSet):
-            order_by_field = 'order'
-
-            class Meta:
-                model = User
-                fields = ['username', 'status']
-                order_by = ['status']
-
-        f = F().form
-        self.assertNotIn('o', f.fields)
-        self.assertIn('order', f.fields)
-        self.assertEqual(f.fields['order'].choices, [('status', 'Status')])
-    
-    def test_ordering_with_overridden_field_name_and_descending(self):
-        """
-        Set the `order_by_field` on the queryset and ensure that the
-        field name is respected.
-        """
-        class F(FilterSet):
-            order_by_field = 'order'
-
-            class Meta:
-                model = User
-                fields = ['username', 'status']
-                order_by = ['status', '-status']
-
-        f = F().form
-        self.assertNotIn('o', f.fields)
-        self.assertIn('order', f.fields)
-        self.assertEqual(f.fields['order'].choices, [('status', 'Status'), ('-status', 'Status (descending)')])
-
-    def test_ordering_with_overridden_field_name_and_using_all_fields(self):
-        class F(FilterSet):
-            order_by_field = 'order'
-
-            class Meta:
-                model = User
-                fields = ['username', 'status']
-                order_by = True
-
-        f = F().form
-        self.assertIn('order', f.fields)
-        self.assertEqual(f.fields['order'].choices,
-            [('username', 'Username'), ('-username', 'Username (descending)'), ('status', 'Status'), ('-status', 'Status (descending)')])
-
-    def test_ordering_with_custom_display_names(self):
-        class F(FilterSet):
-            class Meta:
-                model = User
-                fields = ['username', 'status']
-                order_by = [('status', 'Current status')]
-
+                model = ManagerGroup
+                fields = ['users', 'manager']
         f = F().form
         self.assertEqual(
-            f.fields['o'].choices, [('status', 'Current status')])
+            list(f.fields['users'].choices), [(2, 'active')]
+        )
+        self.assertEqual(
+            list(f.fields['manager'].choices), [('', '---------'), (3, 'manager')]
+        )
+
+    def test_disabled_help_text(self):
+        class F(FilterSet):
+            class Meta:
+                model = Book
+                fields = {
+                    # 'in' lookups are CSV-based, which have a `help_text`.
+                    'title': ['in']
+                }
+
+        self.assertEqual(
+            F().form.fields['title__in'].help_text,
+            'Multiple values may be separated by commas.'
+        )
+
+        with override_settings(FILTERS_DISABLE_HELP_TEXT=True):
+
+            self.assertEqual(
+                F().form.fields['title__in'].help_text,
+                ''
+            )
